//
// Dmitry Friesen: edit
//
#include <idc.idc>

static getCName(ea)
{
	auto name, ch;

	name = "";
	while ( ch = Byte(ea++) )
		name = name + ch;

	return name;
}

static FunctionStart(ea)
{
	// Dmitry Friesen: add
	auto fncEA;
	// end

	fncEA = LocByName(GetFunctionName(ea));
	
	// Dmitry Friesen: add
	if ( fncEA == BADADDR )
	{
		fncEA = ea - 0x15;
		if ( MakeFunction(fncEA, BADADDR) == 0 )
			return BADADDR;
	}
	// end
	return fncEA;
}

// Dmitry Friesen: add
// For double-clicking in output window
static MyMessage(ea, msg)
{
	auto segname;

	segname = SegName(ea);
	if ( segname != "" )
	{
		Message("%s:%08X: %s", segname, ea, msg);
	}
	else
	{
		Message(":%08X: %s", ea, msg);
	}
}
// end

static VBDllCall(functionBody)
{
	auto dll, fnc, fncEa, ea, x, oldfncname, newfncname;
	// Dmitry Friesen: add
	auto dllname, oldprm;

	fncEa = FunctionStart(functionBody);
	if ( fncEa == BADADDR )
	{
		
		// Dmitry Friesen: replace
		MyMessage(functionBody, "Must be called from inside a function!\n");
		// end
		return 0;
	}

	ea = fncEa;

	// skip some instructions
	for ( x = 0; x < 4; x++ )
		ea = NextHead(ea, BADADDR);

	// check if valid signature
	if ( Byte(ea) != 0x68 )
	{
		// Dmitry Friesen: replace
		MyMessage(fncEa, "Not a valid DllFunctionCall()\n");
		// end
		return 0;
	}

	// get the value @ push XXXX
	x = GetOperandValue(ea, 0);

	// point to DLL name
	dll = Dword(x);
	// Dmitry Friesen: add
	dllname = getCName(dll);
	// end

	// point to function name
	fnc = Dword(x + 4);

	// get name of new function
	newfncname = getCName(fnc);

	// get old function name
	oldfncname = GetFunctionName(fncEa);

	// get value of dword ptr: mov eax, dword_xxxxx
	x = GetOperandValue(fncEa, 1);
	if ( (x != BADADDR) && (strstr(Name(x), "dword") != -1) )
	{
		// rename the 'dword' that points to the API
		// Dmitry Friesen: edit
		MakeDword(x);
		MakeName(x, "pfn_" + newfncname);
		MakeDword(x - 8);
		MakeDword(x - 4);
		MakeName(x - 8, "hLib_" + dllname + newfncname);
		// end
	}

	// check if already named something similar to the new name
	if ( strstr(oldfncname, newfncname) >= 0 )
	{
		// Dmitry Friesen: replace
		MyMessage(fncEa, form("%s is already named as something relevant!\n", oldfncname));
		// end
		return 0;
	}

	// Dmitry Friesen add
	// For correct MakeStr(...)
	oldprm = GetLongPrm(INF_STRTYPE);
	SetLongPrm(INF_STRTYPE, ASCSTR_C);
	// end
	MakeUnkn(dll, 0);
	MakeStr(dll, BADADDR);

	MakeUnkn(fnc, 0);
	MakeStr(fnc, BADADDR);
	// Dmitry Friesen add
	SetLongPrm(INF_STRTYPE, oldprm);
	// end

	MakeName(fncEa, newfncname);

	// Dmitry Friesen: replace
	MyMessage(fncEa, form("%s is renamed to %s\n", oldfncname, newfncname));
	// end
	return 1;
}

static main()
{
	auto x, y, c, ea;

	x = LocByName("DllFunctionCall");

	if ( x == BADADDR )
	{
		Message("No reference to VB's DllFunctionCall()...script aborted...\n");
		return;
	}

	Message("VBDllCall v0.1 is searching for functions to be renamed....\n");
	c = 0;
	for ( y=RfirstB(x); y != BADADDR; y = RnextB(x,y) )
	{
		if ( XrefType() != fl_CN )
			continue;
		if ( VBDllCall(y) )
			c++;
	}
	Message(ltoa(c, 10) + " VB Dll function calls encountered and named appropriately!\n");
}
